@@ -24,7 +24,7 @@ import gpxviewer.rc_gpxviewer
 def main():
   app = QApplication(sys.argv)
   QCoreApplication.setApplicationName('GPX Viewer')
-  QCoreApplication.setApplicationVersion('1.0')
+  QCoreApplication.setApplicationVersion('1.1')
 
   translator = QTranslator(app)
   translator.load(QLocale(), 'gpxviewer', '_', ':/translations')
@@ -36,9 +36,9 @@ def main():
   gpxMainWindow = GpxMainWindow()
   gpxMainWindow.show()
   if len(sys.argv) > 1:
-    if sys.argv[1].endswith('.gpx'):
+    if sys.argv[1].lower().endswith('.gpx'):
       gpxMainWindow.openGPXFiles(sys.argv[1:])
-    if sys.argv[1].endswith('.gpxv'):
+    else:
       gpxMainWindow.openGPXProject(sys.argv[1])
   sys.exit(app.exec_())
 
