@@ -15,13 +15,21 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import configparser
+import os
+import shutil
+from tempfile import mkdtemp
+import zipfile
 from PyQt5.QtCore import Qt, QCoreApplication, QFileInfo, QObject, pyqtSignal
 import gpxviewer.gpxmodel as gpx
 from gpxviewer.configstore import GpxConfigParser, TheConfig
 
 
+__all__ = ['TheDocument', 'TYPE_GPXV', 'TYPE_GPXZ']
+
+
 GPXMAGICK = '9e27ea8e'
 FORMAT_VERSION = 2
+TYPE_GPXV, TYPE_GPXZ = range(2)
 
 
 class GpxDocument(QObject):
@@ -30,51 +38,103 @@ class GpxDocument(QObject):
     self.gpxparser = gpx.GpxParser()
     self.wptmodel = self.gpxparser.wptmodel
     self.trkmodel = self.gpxparser.trkmodel
+    self.tmpdir = mkdtemp(prefix='gpxviewer_tmpdir.')
+    self.projectType = TYPE_GPXV
     self.doc = {}
     self.doc['GPXFile'] = []
 
+  def close(self):
+    shutil.rmtree(self.tmpdir)
+
   def saveFile(self, filename):
-    with open(filename, 'w', encoding='utf-8') as file:
-      self.doc['FormatVersion'] = FORMAT_VERSION
+    self.doc['FormatVersion'] = FORMAT_VERSION
 
-      # Number of points/tracks to check the validity of the files
-      self.doc['NumberOfPoints'] = self.wptmodel.rowCount()
-      self.doc['NumberOfTracks'] = self.trkmodel.rowCount()
+    # Number of points/tracks to check the validity of the files
+    self.doc['NumberOfPoints'] = self.wptmodel.rowCount()
+    self.doc['NumberOfTracks'] = self.trkmodel.rowCount()
 
-      self.doc['SkipPoints'] = self.wptmodel.getSkippedPoints()
-      self.doc['MarkerPoints'] = self.wptmodel.getMarkedPoints()
-      self.doc['CaptionPoints'] = self.wptmodel.getCaptionedPoints()
-      self.doc['SplitLines'] = self.wptmodel.getSplitLines()
-      self.doc['NeglectDistances'] = self.wptmodel.getNeglectStates()
-      self.doc['SkipTracks'] = self.trkmodel.getSkippedTracks()
+    self.doc['SkipPoints'] = self.wptmodel.getSkippedPoints()
+    self.doc['MarkerPoints'] = self.wptmodel.getMarkedPoints()
+    self.doc['CaptionPoints'] = self.wptmodel.getCaptionedPoints()
+    self.doc['SplitLines'] = self.wptmodel.getSplitLines()
+    self.doc['NeglectDistances'] = self.wptmodel.getNeglectStates()
+    self.doc['SkipTracks'] = self.trkmodel.getSkippedTracks()
 
-      self.doc.update(TheConfig['ProfileStyle'])
+    self.doc.update(TheConfig['ProfileStyle'])
 
-      self.doc['MarkerColors'] = self.wptmodel.getPointStyles(gpx.MARKER_COLOR)
-      self.doc['MarkerStyles'] = self.wptmodel.getPointStyles(gpx.MARKER_STYLE)
-      self.doc['MarkerSizes'] = self.wptmodel.getPointStyles(gpx.MARKER_SIZE)
+    self.doc['MarkerColors'] = self.wptmodel.getPointStyles(gpx.MARKER_COLOR)
+    self.doc['MarkerStyles'] = self.wptmodel.getPointStyles(gpx.MARKER_STYLE)
+    self.doc['MarkerSizes'] = self.wptmodel.getPointStyles(gpx.MARKER_SIZE)
 
-      self.doc['SplitLineColors'] = self.wptmodel.getPointStyles(gpx.LINE_COLOR)
-      self.doc['SplitLineStyles'] = self.wptmodel.getPointStyles(gpx.LINE_STYLE)
-      self.doc['SplitLineWidths'] = self.wptmodel.getPointStyles(gpx.LINE_WIDTH)
+    self.doc['SplitLineColors'] = self.wptmodel.getPointStyles(gpx.LINE_COLOR)
+    self.doc['SplitLineStyles'] = self.wptmodel.getPointStyles(gpx.LINE_STYLE)
+    self.doc['SplitLineWidths'] = self.wptmodel.getPointStyles(gpx.LINE_WIDTH)
 
-      self.doc['CaptionPositionXs'] = self.wptmodel.getPointStyles(gpx.CAPTION_POSX)
-      self.doc['CaptionPositionYs'] = self.wptmodel.getPointStyles(gpx.CAPTION_POSY)
-      self.doc['CaptionSizes'] = self.wptmodel.getPointStyles(gpx.CAPTION_SIZE)
+    self.doc['CaptionPositionXs'] = self.wptmodel.getPointStyles(gpx.CAPTION_POSX)
+    self.doc['CaptionPositionYs'] = self.wptmodel.getPointStyles(gpx.CAPTION_POSY)
+    self.doc['CaptionSizes'] = self.wptmodel.getPointStyles(gpx.CAPTION_SIZE)
 
-      self.doc['ChangedNames'] = list(self.wptmodel.changedNames.keys())
-      self.doc['PointNames'] = list(self.wptmodel.changedNames.values())
+    self.doc['ChangedNames'] = list(self.wptmodel.changedNames.keys())
+    self.doc['PointNames'] = list(self.wptmodel.changedNames.values())
+
+    cfg = GpxConfigParser()
+
+    if self.projectType == TYPE_GPXV:
+      # Check if we want to extract some GPX files
+      self.doExtractFiles = False
+      if any([QFileInfo(gpxfile).isRelative() for gpxfile in self.doc['GPXFile']]):
+        self.askToExtractFiles.emit([gpxfile for gpxfile in self.doc['GPXFile'] if QFileInfo(gpxfile).isRelative()])
+
+      for i, gpxfile in enumerate(self.doc['GPXFile']):
+        if QFileInfo(gpxfile).isRelative() and self.doExtractFiles:
+          newgpxfile = os.path.join(QFileInfo(filename).path(), gpxfile)
+          shutil.copy(os.path.join(self.tmpdir, gpxfile), newgpxfile)
+          self.doc['GPXFile'][i] = newgpxfile
 
-      cfg = GpxConfigParser()
       cfg.read_dict({GPXMAGICK: self.doc})
-      cfg.write(file)
+      with open(filename, 'w', encoding='utf-8') as file:
+        cfg.write(file)
+
+    else:  # TYPE_GPXZ
+      with zipfile.ZipFile(filename, mode='w', compression=zipfile.ZIP_DEFLATED) as archive:
+        for gpxfile in self.doc['GPXFile']:
+          if QFileInfo(gpxfile).isRelative():  # make absolute path
+            gpxfile = os.path.join(self.tmpdir, gpxfile)
+          archive.write(gpxfile, arcname=QFileInfo(gpxfile).fileName())
+
+        # Make all paths relative
+        self.doc['GPXFile'] = [QFileInfo(gpxfile).fileName() for gpxfile in self.doc['GPXFile']]
+        cfg.read_dict({GPXMAGICK: self.doc})
+        with open(os.path.join(self.tmpdir, 'doc.gpxv'), 'w', encoding='utf-8') as file:
+          cfg.write(file)
+          file.flush()
+          archive.write(file.name, arcname='doc.gpxv')
 
   def openFile(self, filename):
     cfg = GpxConfigParser()
+    curFileName = filename
+
+    if filename.lower().endswith('.gpxv'):
+      self.projectType = TYPE_GPXV
+    else:
+      try:
+        tmpfile = os.path.join(self.tmpdir, 'doc.gpxv')
+        # We should remove older version of doc.gpxv
+        if QFileInfo(tmpfile).exists():
+          os.remove(tmpfile)
+        with zipfile.ZipFile(filename, mode='r', compression=zipfile.ZIP_DEFLATED) as archive:
+          archive.extractall(self.tmpdir)
+          curFileName = tmpfile
+          if not QFileInfo(curFileName).exists():
+            raise gpx.GpxWarning(QCoreApplication.translate('GpxDocument', 'This file is not a valid GPX Viewer archive.'))
+        self.projectType = TYPE_GPXZ
+      except zipfile.BadZipFile:  # filename is not a ZIP archive
+        self.projectType = TYPE_GPXV
+
     try:
-      cfg.read(filename, encoding='utf-8')
+      cfg.read(curFileName, encoding='utf-8')
     except (configparser.ParsingError, UnicodeDecodeError):
-      raise gpx.GpxWarning(QCoreApplication.translate('GpxDocument', 'This file in not a valid GPX Viewer project file.'))
+      raise gpx.GpxWarning(QCoreApplication.translate('GpxDocument', 'This file is not a valid GPX Viewer project.'))
 
     if GPXMAGICK in cfg:
       self.doc.clear()
@@ -91,18 +151,21 @@ class GpxDocument(QObject):
 
       self.applyToAll = False
       for i, file in enumerate(self.doc['GPXFile']):
-        if not QFileInfo(file).exists():
-          self.newFilePath = ''
-          self.fileNotFound.emit(file)
-          if self.newFilePath != '':
-            self.doc['GPXFile'][i] = self.newFilePath
-            file = self.newFilePath
-            if self.applyToAll:
-              for j in range(i + 1, len(self.doc['GPXFile'])):
-                self.doc['GPXFile'][j] = self.doc['GPXFile'][j].replace(QFileInfo(self.doc['GPXFile'][j]).path(), QFileInfo(self.newFilePath).path())
-          else:
-            raise gpx.GpxWarning(QCoreApplication.translate('GpxDocument', 'One of GPX files doesn\'t exist. This project can\'t be opened.'))
-        self.gpxparser.parse(file)
+        if self.projectType == TYPE_GPXV:
+          if not QFileInfo(file).exists():
+            self.newFilePath = ''
+            self.fileNotFound.emit(file)
+            if self.newFilePath != '':
+              self.doc['GPXFile'][i] = self.newFilePath
+              file = self.newFilePath
+              if self.applyToAll:
+                for j in range(i + 1, len(self.doc['GPXFile'])):
+                  self.doc['GPXFile'][j] = self.doc['GPXFile'][j].replace(QFileInfo(self.doc['GPXFile'][j]).path(), QFileInfo(self.newFilePath).path())
+            else:
+              raise gpx.GpxWarning(QCoreApplication.translate('GpxDocument', 'One of GPX files doesn\'t exist. This project can\'t be opened.'))
+          self.gpxparser.parse(file)
+        else:  # TYPE_GPXZ
+          self.gpxparser.parse(os.path.join(self.tmpdir, file))
 
       if ('NumberOfPoints' in self.doc and self.wptmodel.rowCount() != int(self.doc['NumberOfPoints'])) or \
          ('NumberOfTracks' in self.doc and self.trkmodel.rowCount() != int(self.doc['NumberOfTracks'])):
@@ -134,12 +197,18 @@ class GpxDocument(QObject):
         TheConfig['ProfileStyle']['MinimumAltitude'] = self.doc['MinimumAltitude']
       if 'MaximumAltitude' in self.doc:
         TheConfig['ProfileStyle']['MaximumAltitude'] = self.doc['MaximumAltitude']
+      if 'SelectedPointsOnly' in self.doc:
+        TheConfig['ProfileStyle']['SelectedPointsOnly'] = self.doc['SelectedPointsOnly']
+      if 'AutoscaleAltitudes' in self.doc:
+        TheConfig['ProfileStyle']['AutoscaleAltitudes'] = self.doc['AutoscaleAltitudes']
+      if 'FontSize' in self.doc:
+        TheConfig['ProfileStyle']['FontSize'] = self.doc['FontSize']
+      if 'FontFamily' in self.doc:
+        TheConfig['ProfileStyle']['FontFamily'] = self.doc['FontFamily']
       if 'DistanceCoefficient' in self.doc:
         TheConfig['ProfileStyle']['DistanceCoefficient'] = self.doc['DistanceCoefficient']
       if 'TimeZoneOffset' in self.doc:
         TheConfig['ProfileStyle']['TimeZoneOffset'] = self.doc['TimeZoneOffset']
-      if 'SelectedPointsOnly' in self.doc:
-        TheConfig['ProfileStyle']['SelectedPointsOnly'] = self.doc['SelectedPointsOnly']
       if 'ReadNameFromTag' in self.doc:
         TheConfig['ProfileStyle']['ReadNameFromTag'] = self.doc['ReadNameFromTag']
       if 'CoordinateFormat' in self.doc:
@@ -194,9 +263,10 @@ class GpxDocument(QObject):
         for i, n in zip(self.doc['ChangedNames'], self.doc['PointNames']):
           self.wptmodel.setData(self.wptmodel.index(i, gpx.NAME), n, Qt.EditRole)
     else:  # GPXMAGICK not in cfg
-      raise gpx.GpxWarning(QCoreApplication.translate('GpxDocument', 'This file in not a valid GPX Viewer project file.'))
+      raise gpx.GpxWarning(QCoreApplication.translate('GpxDocument', 'This file is not a valid GPX Viewer project.'))
 
   fileNotFound = pyqtSignal(str)
+  askToExtractFiles = pyqtSignal(list)
 
 
 TheDocument = GpxDocument()
