@@ -25,7 +25,7 @@ import gpxviewer.gpxmodel as gpx
 import gpxviewer.statwindow as stat
 import gpxviewer.plotviewer as plt
 from gpxviewer.configstore import TheConfig
-from gpxviewer.gpxdocument import TheDocument
+from gpxviewer.gpxdocument import *
 from gpxviewer.pointconfigdialog import PointConfigDialog
 from gpxviewer.settingsdialog import SettingsDialog
 import gpxviewer.ui_mainwindow
@@ -47,6 +47,7 @@ class GpxMainWindow(QMainWindow):
     self.ui.actionSaveAs.setIcon(QIcon(':/icons/document-save-as.svg'))
     self.ui.menuRecentProjects.setIcon(QIcon(':/icons/document-open-recent.svg'))
     self.ui.actionQuit.setIcon(QIcon(':/icons/application-exit.svg'))
+    self.ui.actionCopy.setIcon(QIcon(':/icons/edit-copy.svg'))
     self.ui.actionSettings.setIcon(QIcon(':/icons/configure.svg'))
     self.ui.actionDistanceProfile.setIcon(QIcon(':/icons/distanceprofile.svg'))
     self.ui.actionTimeProfile.setIcon(QIcon(':/icons/timeprofile.svg'))
@@ -60,6 +61,9 @@ class GpxMainWindow(QMainWindow):
     self.ui.actionSave.setShortcut(QKeySequence.Save)
     self.ui.actionSaveAs.setShortcut(QKeySequence.SaveAs)
     self.ui.actionQuit.setShortcut(QKeySequence.Quit)
+    self.ui.actionCopy.setShortcut(QKeySequence.Copy)
+    self.ui.actionSettings.setShortcut(QKeySequence.Preferences)
+    self.ui.actionGpxViewerHelp.setShortcut(QKeySequence.HelpContents)
 
     wdg = QWidget()
     wdg.setLayout(QHBoxLayout())
@@ -96,20 +100,24 @@ class GpxMainWindow(QMainWindow):
     TheDocument.gpxparser.warningSent[str, str].connect(self.showWarning)
     TheDocument.wptmodel.wptDataChanged.connect(self.setProjectChanged)
     TheDocument.fileNotFound[str].connect(self.openedFileNotFound)
+    TheDocument.askToExtractFiles[list].connect(self.askedExtractFiles)
 
     self.projectSaved = False
     self.projectChanged = False
     self.titleFilename = None
     self.actionsRecent = []
+    self.actionsColumns = []
 
     self.updateRecentProjects()
+    self.initColumnsToCopy()
     self.resize(TheConfig['MainWindow'].getint('WindowWidth'), TheConfig['MainWindow'].getint('WindowHeight'))
 
     self.plot = plt.PlotWindow()
     self.plot.profileChanged.connect(self.setProjectChanged)
     self.stat = stat.StatWindow()
 
-  def aboutQt(self):
+  @pyqtSlot()
+  def onAboutQt(self):
     QApplication.aboutQt()
 
   def eventFilter(self, obj, event):
@@ -134,7 +142,7 @@ class GpxMainWindow(QMainWindow):
                                        self.tr('There are unsaved changes. Do you want to save the project?'),
                                        QMessageBox.Yes | QMessageBox.No | QMessageBox.Cancel, QMessageBox.Yes)
       if result == QMessageBox.Yes:
-        if not self.fileSave():
+        if not self.onFileSave():
           event.ignore()
           return
       if result == QMessageBox.Cancel:
@@ -143,6 +151,7 @@ class GpxMainWindow(QMainWindow):
 
     self.plot.close()
     self.stat.close()
+    TheDocument.close()
     TheConfig.save()
     super(GpxMainWindow, self).closeEvent(event)
 
@@ -262,11 +271,6 @@ class GpxMainWindow(QMainWindow):
     if event.key() == Qt.Key_F and event.modifiers() == Qt.ControlModifier:
       self.filterLineEdit.setFocus()
       self.filterLineEdit.selectAll()
-    elif event.key() == Qt.Key_C and event.modifiers() == Qt.ControlModifier:
-      if self.ui.tabWidget.currentWidget() == self.ui.wptTab:
-        TheDocument.wptmodel.copyToClipboard([i.data(gpx.IDRole) for i in self.ui.wptView.selectionModel().selectedRows()])
-      else:
-        TheDocument.trkmodel.copyToClipboard([i.row() for i in self.ui.trkView.selectionModel().selectedRows()])
     elif event.key() == Qt.Key_F2:
       if self.ui.tabWidget.currentWidget() == self.ui.wptTab and self.ui.wptView.selectionModel().hasSelection():
         self.renamePoints()
@@ -293,7 +297,8 @@ class GpxMainWindow(QMainWindow):
     TheConfig['MainWindow']['WindowWidth'] = str(event.size().width())
     TheConfig['MainWindow']['WindowHeight'] = str(event.size().height())
 
-  def aboutGPXViewer(self):
+  @pyqtSlot()
+  def onAboutGPXViewer(self):
     aboutText = '<h3>GPX Viewer</h3><b>' + self.tr('Version') + ' ' + QCoreApplication.applicationVersion() + '</b><br><br>' + \
                 self.tr('Using') + ' Python ' + str(sys.version_info.major) + '.' + str(sys.version_info.minor) + '.' + str(sys.version_info.micro) + ', ' + \
                 'PyQt5 ' + PYQT_VERSION_STR + ', ' + \
@@ -302,7 +307,8 @@ class GpxMainWindow(QMainWindow):
                 self.tr('License:') + ' <a href=http://www.gnu.org/licenses/gpl.html>GNU General Public License, version 3</a>'
     QMessageBox.about(self, self.tr('About GPX Viewer'), aboutText)
 
-  def gpxViewerHelp(self):
+  @pyqtSlot()
+  def onGpxViewerHelp(self):
     aboutText = self.tr('''Notation:<br>
                            <font color=red>Red</font> - skipped points<br>
                            <font color=blue>Blue</font> - marked points<br>
@@ -310,11 +316,6 @@ class GpxMainWindow(QMainWindow):
                            <b>Bold</b> - points with splitting lines<br>
                            <i>Italic</i> - distance before these points is neglected<br>
                            <br>
-                           Useful shortcuts:<br>
-                           Ctrl+C - Copy points to clipboard<br>
-                           Ctrl+P - Show distance profile<br>
-                           Ctrl+T - Show time profile<br>
-                           Ctrl+I - Show statistics<br>
                            F2 - Rename multiple points.
                            Several symbols "#" are replaced by sequential numbers.
                            The number of digits equals to the amount of symbols "#".<br>
@@ -473,25 +474,28 @@ class GpxMainWindow(QMainWindow):
                                           not self.sender().isChecked())
     self.setProjectChanged()
 
-  def fileNew(self):
+  @pyqtSlot()
+  def onFileNew(self):
     if self.projectChanged and len(TheDocument.doc['GPXFile']) != 0:
       result = QMessageBox.information(self, self.tr('Load GPX file'),
                                        self.tr('There are unsaved changes. Do you want to save the project?'),
                                        QMessageBox.Yes | QMessageBox.No | QMessageBox.Cancel, QMessageBox.Yes)
       if result == QMessageBox.Yes:
-        if not self.fileSave():
+        if not self.onFileSave():
           return
       if result == QMessageBox.Cancel:
         return
 
     self.reset()
 
-  def fileLoadGPXFile(self):
+  @pyqtSlot()
+  def onFileLoadGPXFile(self):
     filenames = QFileDialog.getOpenFileNames(self, self.tr('Open GPX file'), TheConfig['MainWindow']['LoadGPXDirectory'],
                                              self.tr('GPX XML (*.gpx *.GPX);;All files (*)'))[0]
     self.openGPXFiles(filenames)
 
-  def fileSaveGPXFileAs(self):
+  @pyqtSlot()
+  def onFileSaveGPXFileAs(self):
     if TheDocument.wptmodel.rowCount() == len(TheDocument.wptmodel.getSkippedPoints()) and \
        TheDocument.trkmodel.rowCount() == len(TheDocument.trkmodel.getSkippedTracks()):
       QMessageBox.warning(self, self.tr('Save error'), self.tr('The GPX file will be empty.'))
@@ -502,26 +506,32 @@ class GpxMainWindow(QMainWindow):
     if filename != '':
       TheConfig['MainWindow']['LoadGPXDirectory'] = QFileInfo(filename).path()
       TheDocument.gpxparser.writeToFile(filename)
+      self.ui.statusBar.showMessage(filename + self.tr(' written.'), 2000)
 
-  def fileOpen(self):
+  @pyqtSlot()
+  def onFileOpen(self):
     if self.projectChanged and len(TheDocument.doc['GPXFile']) != 0:
       result = QMessageBox.information(self, self.tr('Open GPX Viewer project'),
                                        self.tr('There are unsaved changes. Do you want to save the project?'),
                                        QMessageBox.Yes | QMessageBox.No | QMessageBox.Cancel, QMessageBox.Yes)
       if result == QMessageBox.Yes:
-        if not self.fileSave():
+        if not self.onFileSave():
           return
       if result == QMessageBox.Cancel:
         return
 
     filename = QFileDialog.getOpenFileName(self, self.tr('Open project file'), TheConfig['MainWindow']['ProjectDirectory'],
-                                           self.tr('GPX Viewer Projects (*.gpxv *.GPXV);;All files (*)'))[0]
+                                           self.tr('All GPX Viewer files (*.gpxv *.gpxz *.GPXV *.GPXZ);;'
+                                                   'GPX Viewer projects (*.gpxv *.GPXV);;'
+                                                   'GPX Viewer archives (*.gpxz *.GPXZ);;'
+                                                   'All files (*)'))[0]
     if filename != '':
       self.openGPXProject(filename)
 
-  def fileSave(self):
+  @pyqtSlot()
+  def onFileSave(self):
     if not self.projectSaved:
-      return self.fileSaveAs()
+      return self.onFileSaveAs()
     else:
       try:
         TheDocument.saveFile(self.projectFile)
@@ -529,21 +539,29 @@ class GpxMainWindow(QMainWindow):
         QMessageBox.warning(self, self.tr('Save error'), e.args[0])
         return False
     self.setProjectChanged(False)
+    self.ui.statusBar.showMessage(self.projectFile + self.tr(' saved.'), 2000)
     return True
 
-  def fileSaveAs(self):
+  @pyqtSlot()
+  def onFileSaveAs(self):
     if len(TheDocument.doc['GPXFile']) != 0:
-      filename = QFileDialog.getSaveFileName(self, self.tr('Save project file as'), TheConfig['MainWindow']['ProjectDirectory'],
-                                             self.tr('GPX Viewer Projects (*.gpxv *.GPXV);;All files (*)'))[0]
+      filename, filter = QFileDialog.getSaveFileName(self, self.tr('Save project file as'), TheConfig['MainWindow']['ProjectDirectory'],
+                                                     self.tr('GPX Viewer projects (*.gpxv *.GPXV);;'
+                                                             'GPX Viewer archives (*.gpxz *.GPXZ);;'
+                                                             'All files (*)'),
+                                                     TheConfig['MainWindow']['ProjectExtension'])
       if filename != '':
+        TheDocument.projectType = TYPE_GPXZ if (filter.find('gpxz') != -1 or filename.lower().endswith('.gpxz')) else TYPE_GPXV
         try:
           TheDocument.saveFile(filename)
           self.projectFile = filename
           self.projectSaved = True
           TheConfig['MainWindow']['ProjectDirectory'] = QFileInfo(self.projectFile).path()
+          TheConfig['MainWindow']['ProjectExtension'] = filter
           self.setProjectChanged(False)
           self.updateTitleFilename(self.projectFile)
           self.addRecentProject(self.projectFile)
+          self.ui.statusBar.showMessage(self.projectFile + self.tr(' saved.'), 2000)
           return True
         except OSError as e:
           QMessageBox.warning(self, self.tr('Save error'), e.args[0])
@@ -559,6 +577,8 @@ class GpxMainWindow(QMainWindow):
       self.openGPXFile(f)
     TheDocument.gpxparser.updatePoints()
     self.updateTabs()
+    message = (filenames[0] if len(filenames) == 1 else self.tr('Multiple GPX files')) + self.tr(' loaded.', '', len(filenames))
+    self.ui.statusBar.showMessage(message, 2000)
 
   def openGPXFile(self, filename):
     TheConfig['MainWindow']['LoadGPXDirectory'] = QFileInfo(filename).path()
@@ -587,6 +607,7 @@ class GpxMainWindow(QMainWindow):
       self.updateTitleFilename(self.projectFile)
       self.updateTabs()
       self.addRecentProject(self.projectFile)
+      self.ui.statusBar.showMessage(self.projectFile + self.tr(' opened.'), 2000)
     except gpx.GpxWarning as e:
       self.reset()
       QMessageBox.warning(self, self.tr('File read error'), e.args[0])
@@ -604,6 +625,16 @@ class GpxMainWindow(QMainWindow):
         if result == QMessageBox.YesToAll:
           TheDocument.applyToAll = True
 
+  @pyqtSlot(list)
+  def askedExtractFiles(self, files):
+    result = QMessageBox.warning(self, self.tr('Extract files?'), self.tr('This project refers to files that could be missing from the filesystem:\n')
+                                                                  + '\n'.join(files)
+                                                                  + self.tr('\n\nDo you want to write these files along with the saved project?\n'
+                                                                            'NOTE: Some files can be overwritten.'),
+                                 QMessageBox.Yes | QMessageBox.No, QMessageBox.Yes)
+    if result == QMessageBox.Yes:
+      TheDocument.doExtractFiles = True
+
   @pyqtSlot()
   def openRecentProject(self):
     if self.projectChanged and len(TheDocument.doc['GPXFile']) != 0:
@@ -611,7 +642,7 @@ class GpxMainWindow(QMainWindow):
                                        self.tr('There are unsaved changes. Do you want to save the project?'),
                                        QMessageBox.Yes | QMessageBox.No | QMessageBox.Cancel, QMessageBox.Yes)
       if result == QMessageBox.Yes:
-        if not self.fileSave():
+        if not self.onFileSave():
           return
       if result == QMessageBox.Cancel:
         return
@@ -629,7 +660,20 @@ class GpxMainWindow(QMainWindow):
             del TheConfig.recentProjects[i]
             self.updateRecentProjects()
 
-  def plotDistanceProfile(self):
+  @pyqtSlot()
+  def onEditCopy(self):
+    if self.ui.tabWidget.currentWidget() == self.ui.wptTab:
+      TheDocument.wptmodel.copyToClipboard([i.data(gpx.IDRole) for i in self.ui.wptView.selectionModel().selectedRows()])
+    else:
+      TheDocument.trkmodel.copyToClipboard([i.row() for i in self.ui.trkView.selectionModel().selectedRows()])
+
+  @pyqtSlot()
+  def onResetColumns(self):
+    TheConfig.columnsToCopy = list(gpx.WPTFIELDS)
+    self.updateColumnsToCopy()
+
+  @pyqtSlot()
+  def onPlotDistanceProfile(self):
     if TheDocument.wptmodel.rowCount() - len(TheDocument.wptmodel.getSkippedPoints()) < 2 and \
        TheDocument.trkmodel.rowCount() == len(TheDocument.trkmodel.getSkippedTracks()):
       QMessageBox.warning(self, self.tr('Plot error'), self.tr('Not enouph points or tracks.'))
@@ -643,7 +687,8 @@ class GpxMainWindow(QMainWindow):
       self.plot.plotProfile(gpx.DIST)
     self.plot.activateWindow()
 
-  def plotTimeProfile(self):
+  @pyqtSlot()
+  def onPlotTimeProfile(self):
     # Check if there are at least two points with timestamps
     n = 0
     for i in range(TheDocument.wptmodel.rowCount()):
@@ -668,7 +713,8 @@ class GpxMainWindow(QMainWindow):
       self.plot.plotProfile(gpx.TIME_DAYS)
     self.plot.activateWindow()
 
-  def showStatistics(self):
+  @pyqtSlot()
+  def onShowStatistics(self):
     if TheDocument.wptmodel.rowCount() - len(TheDocument.wptmodel.getSkippedPoints()) < 2:
       QMessageBox.warning(self, self.tr('Statistics error'), self.tr('Not enouph points.'))
       return
@@ -676,34 +722,41 @@ class GpxMainWindow(QMainWindow):
     self.stat.show()
     self.stat.activateWindow()
 
-  def showSkipped(self, show):
+  @pyqtSlot(bool)
+  def onShowSkipped(self, show):
     TheConfig['MainWindow']['ShowSkipped'] = str(show)
     self.updateIncludeFilter()
 
-  def showMarked(self, show):
+  @pyqtSlot(bool)
+  def onShowMarked(self, show):
     TheConfig['MainWindow']['ShowMarked'] = str(show)
     self.updateIncludeFilter()
 
-  def showCaptioned(self, show):
+  @pyqtSlot(bool)
+  def onShowCaptioned(self, show):
     TheConfig['MainWindow']['ShowCaptioned'] = str(show)
     self.updateIncludeFilter()
 
-  def showMarkedCaptioned(self, show):
+  @pyqtSlot(bool)
+  def onShowMarkedCaptioned(self, show):
     TheConfig['MainWindow']['ShowMarkedCaptioned'] = str(show)
     self.updateIncludeFilter()
 
-  def showOther(self, show):
+  @pyqtSlot(bool)
+  def onShowOther(self, show):
     TheConfig['MainWindow']['ShowDefault'] = str(show)
     self.updateIncludeFilter()
 
-  def resetFilters(self):
+  @pyqtSlot()
+  def onResetFilters(self):
     self.ui.actionShowSkipped.setChecked(True)
     self.ui.actionShowMarked.setChecked(True)
     self.ui.actionShowCaptioned.setChecked(True)
     self.ui.actionShowMarkedCaptioned.setChecked(True)
     self.ui.actionShowOther.setChecked(True)
 
-  def showSettings(self):
+  @pyqtSlot()
+  def onShowSettings(self):
     dlg = SettingsDialog(self)
     if dlg.exec_() == QDialog.Accepted:
       self.setProjectChanged()
@@ -773,9 +826,34 @@ class GpxMainWindow(QMainWindow):
     else:
       self.ui.actionClearList.setDisabled(True)
 
-  def clearRecentList(self):
+  @pyqtSlot()
+  def onClearRecentList(self):
     for act in self.actionsRecent:
       self.ui.menuRecentProjects.removeAction(act)
     self.actionsRecent = []
     TheConfig.recentProjects = []
     self.ui.actionClearList.setDisabled(True)
+
+  def initColumnsToCopy(self):
+    for i, f in enumerate(TheDocument.wptmodel.fields):
+      act = QAction(f, self)
+      act.setCheckable(True)
+      act.triggered[bool].connect(self.columnsToCopyChanged)
+      self.actionsColumns += [act]
+    self.ui.menuCoLumns.insertActions(self.ui.actionResetColumns, self.actionsColumns)
+    self.ui.menuCoLumns.insertSeparator(self.ui.actionResetColumns)
+    self.updateColumnsToCopy()
+
+  def updateColumnsToCopy(self):
+    for i, act in enumerate(self.actionsColumns):
+      act.setChecked(i in TheConfig.columnsToCopy)
+
+  @pyqtSlot(bool)
+  def columnsToCopyChanged(self, checked):
+    for i, act in enumerate(self.actionsColumns):
+      if self.sender() == act:
+        if checked:
+          TheConfig.columnsToCopy += [i]
+          TheConfig.columnsToCopy.sort()
+        else:
+          TheConfig.columnsToCopy.remove(i)
